var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { Observable } from "rxjs";
import { LocalStorageService } from "./LocalStoreService";
var FirestoreService = /** @class */ (function () {
    function FirestoreService(angularFirestore) {
        this.angularFirestore = angularFirestore;
    }
    FirestoreService.prototype.getRecord = function (collection, uid) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.angularFirestore.collection(collection).doc(uid).get().toPromise().then(function (snapshot) {
                var record = snapshot.data();
                LocalStorageService.setProperty(snapshot.id, record);
                resolve(record);
            }).catch(reject);
        });
    };
    FirestoreService.prototype.addOrUpdateRecord = function (collection, record) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (record.uid) {
                return _this.updateRecord(collection, record).then(function (rec) {
                    LocalStorageService.setProperty(rec.uid, rec);
                    resolve(rec);
                }).catch(reject);
            }
            else {
                return _this.addRecord(collection, record).then(function (rec) {
                    LocalStorageService.setProperty(rec.uid, rec);
                    resolve(rec);
                }).catch(reject);
            }
        });
    };
    FirestoreService.prototype.removeRecord = function (collection, uid) {
        return this.angularFirestore.collection(collection).doc(uid).delete();
    };
    FirestoreService.prototype.observeRecord = function (collection, uid) {
        var _this = this;
        return new Observable(function (observer) {
            _this.angularFirestore.collection(collection).doc(uid).snapshotChanges().subscribe(function (next) {
                var record = next.payload.data();
                LocalStorageService.setProperty(uid, record);
                observer.next(record);
            });
        });
    };
    FirestoreService.prototype.observeCollection = function (collection) {
        var _this = this;
        return new Observable(function (observer) {
            _this.angularFirestore.collection(collection).snapshotChanges().subscribe(function (next) {
                var col = next.map(function (action) { return action.payload.doc.data(); });
                LocalStorageService.setProperty(collection, col);
                observer.next(col);
            });
        });
    };
    FirestoreService.prototype.addRecord = function (collection, record) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.angularFirestore.collection(collection).add(record).then(function (r) {
                record.uid = r.id;
                _this.updateRecord(collection, record).then(resolve).catch(reject);
            });
        });
    };
    FirestoreService.prototype.updateRecord = function (collection, record) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.angularFirestore.collection(collection).doc(record.uid).set(__assign({}, record), { merge: true }).then(function () {
                resolve(record);
            }).catch(reject);
        });
    };
    return FirestoreService;
}());
export { FirestoreService };
//# sourceMappingURL=FirestoreService.js.map