var StoreDataListener = /** @class */ (function () {
    function StoreDataListener() {
    }
    return StoreDataListener;
}());
var StoreDataObserver = /** @class */ (function () {
    function StoreDataObserver() {
        this.listeners = [];
    }
    StoreDataObserver.prototype.observe = function (property, handler) {
        this.listeners.push({
            targetProperty: property,
            handler: handler
        });
    };
    StoreDataObserver.prototype.notify = function (property, value) {
        this.listeners.forEach(function (listener) {
            if (listener.targetProperty === property) {
                listener.handler(value);
            }
        });
    };
    return StoreDataObserver;
}());
var LocalStoreService = /** @class */ (function () {
    function LocalStoreService() {
        this.LOCAL_DATA_NAME = "utils_local_data";
        this.localData = localStorage.getItem(this.LOCAL_DATA_NAME)
            ? JSON.parse(localStorage.getItem(this.LOCAL_DATA_NAME))
            : {};
        this.Observer = new StoreDataObserver();
    }
    LocalStoreService.prototype.getProperty = function (property) {
        return this.localData[property];
    };
    LocalStoreService.prototype.setProperty = function (property, value) {
        this.localData[property] = value;
        localStorage.setItem(this.LOCAL_DATA_NAME, JSON.stringify(this.localData));
        this.Observer.notify(property, value);
    };
    LocalStoreService.prototype.removeProperty = function (property) {
        this.localData[property] = null;
        delete this.localData[property];
        localStorage.setItem(this.LOCAL_DATA_NAME, JSON.stringify(this.localData));
    };
    LocalStoreService.prototype.observe = function (property, handler) {
        this.Observer.observe(property, handler);
    };
    LocalStoreService.prototype.clear = function () {
        this.localData = {};
        localStorage.setItem(this.LOCAL_DATA_NAME, JSON.stringify(this.localData));
    };
    LocalStoreService.prototype.addToArray = function (arrayProperty, object) {
        var array = this.getProperty(arrayProperty);
        array.push(object);
        this.setProperty(arrayProperty, array);
    };
    LocalStoreService.prototype.runOnArray = function (arrayProperty, fn) {
        var array = this.getProperty(arrayProperty);
        array.forEach(function (item) { return fn(item); });
        this.setProperty(arrayProperty, array);
    };
    LocalStoreService.prototype.saveFlashProperty = function (property, value) {
        this.setProperty("flash_" + property, value);
    };
    LocalStoreService.prototype.getFlashProperty = function (property) {
        var propertyValue = this.getProperty("flash_" + property);
        if (propertyValue) {
            propertyValue = JSON.stringify(propertyValue);
            this.removeProperty("flash_" + property);
            return JSON.parse(propertyValue);
        }
        else {
            return null;
        }
    };
    LocalStoreService.prototype.removeFlashProperty = function (property) {
        this.removeProperty("flash_" + property);
    };
    LocalStoreService.prototype.cleanFlashProperties = function () {
        var _this = this;
        Object.keys(this.localData).forEach(function (key) {
            if (key.indexOf("flash_") !== -1) {
                _this.removeProperty(key);
            }
        });
    };
    return LocalStoreService;
}());
export var LocalStorageService = new LocalStoreService();
//# sourceMappingURL=LocalStoreService.js.map