import {AngularFirestore, DocumentChangeAction, DocumentReference, DocumentSnapshot} from '@angular/fire/firestore';
import {Action} from "@angular/fire/database";
import {Observable} from "rxjs";
import {LocalStorageService} from "./LocalStoreService";

export class FirestoreService {

  constructor(private angularFirestore: AngularFirestore) {}

  getRecord(collection: string, uid: string) {
    return new Promise((resolve, reject) => {
      this.angularFirestore.collection(collection).doc(uid).get().toPromise().then(snapshot => {
        let record = snapshot.data();
        LocalStorageService.setProperty(snapshot.id, record);
        resolve(record);
      }).catch(reject);
    });
  }

  addOrUpdateRecord(collection: string, record: any) {
    return new Promise((resolve, reject) => {
      if(record.uid) {
        return this.updateRecord(collection, record).then((rec: any) => {
          LocalStorageService.setProperty(rec.uid, rec);
          resolve(rec);
        }).catch(reject);
      } else {
        return this.addRecord(collection, record).then((rec: any) => {
          LocalStorageService.setProperty(rec.uid, rec);
          resolve(rec);
        }).catch(reject);
      }
    });
  }

  removeRecord(collection: string, uid: string) {
    return this.angularFirestore.collection(collection).doc(uid).delete();
  }

  observeRecord(collection: string, uid: string): Observable<any> {
    return new Observable(observer => {
      this.angularFirestore.collection(collection).doc(uid).snapshotChanges().subscribe((next: Action<DocumentSnapshot<any>>) => {
        let record = next.payload.data();
        LocalStorageService.setProperty(uid, record);
        observer.next(record);
      });
    });
  }

  observeCollection(collection: string): Observable<any> {
    return new Observable(observer => {
      this.angularFirestore.collection(collection).snapshotChanges().subscribe((next: DocumentChangeAction<any>[]) => {
        let col = next.map(action => action.payload.doc.data());
        LocalStorageService.setProperty(collection, col);
        observer.next(col);
      });
    });
  }

  private addRecord(collection, record): Promise<any> {
    return new Promise((resolve, reject) => {
      this.angularFirestore.collection(collection).add(record).then((r: DocumentReference) => {
        record.uid = r.id;
        this.updateRecord(collection, record).then(resolve).catch(reject);
      });
    });
  }

  private updateRecord(collection, record) {
    return new Promise((resolve, reject) => {
      this.angularFirestore.collection(collection).doc(record.uid).set({...record}, {merge: true}).then(() => {
        resolve(record);
      }).catch(reject);
    });
  }
}
